@@ -1,7 +1,7 @@
 /*
  * mdadm - manage Linux "md" devices aka RAID arrays.
  *
- * Copyright (C) 2001-2002 Neil Brown <neilb@cse.unsw.edu.au>
+ * Copyright (C) 2001-2006 Neil Brown <neilb@suse.de>
  *
  *
  *    This program is free software; you can redistribute it and/or modify
@@ -33,6 +33,8 @@
 #include	<glob.h>
 #include	<fnmatch.h>
 #include	<ctype.h>
+#include	<pwd.h>
+#include	<grp.h>
 
 /*
  * Read the config file
@@ -77,7 +79,17 @@
 char DefaultConfFile[] = CONFFILE;
 char DefaultAltConfFile[] = CONFFILE2;
 
-char *keywords[] = { "device", "array", "mailaddr", "program", NULL };
+enum linetype { Devices, Array, Mailaddr, Mailfrom, Program, CreateDev, Homehost, LTEnd };
+char *keywords[] = {
+	[Devices]  = "devices",
+	[Array]    = "array",
+	[Mailaddr] = "mailaddr",
+	[Mailfrom] = "mailfrom",
+	[Program]  = "program",
+	[CreateDev]   = "create",
+	[Homehost] = "homehost",
+	[LTEnd]    = NULL
+};
 
 /*
  * match_keyword returns an index into the keywords array, or -1 for no match
@@ -230,16 +242,135 @@ mddev_dev_t load_partitions(void)
 		minor = strtoul(mp, NULL, 10);
 
 		name = map_dev(major, minor, 1);
-
+		if (!name)
+			continue;
 		d = malloc(sizeof(*d));
 		d->devname = strdup(name);
 		d->next = rv;
+		d->used = 0;
 		rv = d;
 	}
 	fclose(f);
 	return rv;
 }
 
+struct createinfo createinfo = {
+#ifdef DEBIAN
+	.gid = 6, /* disk */
+	.mode = 0660,
+#else
+	.mode = 0600,
+#endif
+};
+
+int parse_auto(char *str, char *msg, int config)
+{
+	int autof;
+	if (str == NULL || *str == 0)
+		autof = 2;
+	else if (strcasecmp(str,"no")==0)
+		autof = 1;
+	else if (strcasecmp(str,"yes")==0)
+		autof = 2;
+	else if (strcasecmp(str,"md")==0)
+		autof = config?5:3;
+	else {
+		/* There might be digits, and maybe a hypen, at the end */
+		char *e = str + strlen(str);
+		int num = 4;
+		int len;
+		while (e > str && isdigit(e[-1]))
+			e--;
+		if (*e) {
+			num = atoi(e);
+			if (num <= 0) num = 1;
+		}
+		if (e > str && e[-1] == '-')
+			e--;
+		len = e - str;
+		if ((len == 2 && strncasecmp(str,"md",2)==0)) {
+			autof = config ? 5 : 3;
+		} else if ((len == 3 && strncasecmp(str,"mdp",3)==0)) {
+			autof = config ? 6 : 4;
+		} else if ((len == 1 && strncasecmp(str,"p",1)==0) ||
+			   (len >= 4 && strncasecmp(str,"part",4)==0)) {
+			autof = 6;
+		} else {
+			fprintf(stderr, Name ": %s arg of \"%s\" unrecognised: use no,yes,md,mdp,part\n"
+				"        optionally followed by a number.\n",
+				msg, str);
+			exit(2);
+		}
+		autof |= num << 3;
+	}
+	return autof;
+}
+
+static void createline(char *line)
+{
+	char *w;
+	char *ep;
+
+	for (w=dl_next(line); w!=line; w=dl_next(w)) {
+		if (strncasecmp(w, "auto=", 5) == 0)
+			createinfo.autof = parse_auto(w+5, "auto=", 1);
+		else if (strncasecmp(w, "owner=", 6) == 0) {
+			if (w[6] == 0) {
+				fprintf(stderr, Name ": missing owner name\n");
+				continue;
+			}
+			createinfo.uid = strtoul(w+6, &ep, 10);
+			if (*ep != 0) {
+				struct passwd *pw;
+				/* must be a name */
+				pw = getpwnam(w+6);
+				if (pw)
+					createinfo.uid = pw->pw_uid;
+				else
+					fprintf(stderr, Name ": CREATE user %s not found\n", w+6);
+			}
+		} else if (strncasecmp(w, "group=", 6) == 0) {
+			if (w[6] == 0) {
+				fprintf(stderr, Name ": missing group name\n");
+				continue;
+			}
+			createinfo.gid = strtoul(w+6, &ep, 10);
+			if (*ep != 0) {
+				struct group *gr;
+				/* must be a name */
+				gr = getgrnam(w+6);
+				if (gr)
+					createinfo.gid = gr->gr_gid;
+				else
+					fprintf(stderr, Name ": CREATE group %s not found\n", w+6);
+			}
+		} else if (strncasecmp(w, "mode=", 5) == 0) {
+			if (w[5] == 0) {
+				fprintf(stderr, Name ": missing CREATE mode\n");
+				continue;
+			}
+			createinfo.mode = strtoul(w+5, &ep, 8);
+			if (*ep != 0) {
+				createinfo.mode = 0600;
+				fprintf(stderr, Name ": unrecognised CREATE mode %s\n",
+					w+5);
+			}
+		} else if (strncasecmp(w, "metadata=", 9) == 0) {
+			/* style of metadata to use by default */
+			int i;
+			for (i=0; superlist[i] && !createinfo.supertype; i++)
+				createinfo.supertype =
+					superlist[i]->match_metadata_desc(w+9);
+			if (!createinfo.supertype)
+				fprintf(stderr, Name ": metadata format %s unknown, ignoring\n",
+					w+9);
+
+		} else {
+			fprintf(stderr, Name ": unrecognised word on CREATE line: %s\n",
+				w);
+		}
+	}
+}
 
 void devline(char *line) 
 {
@@ -273,7 +404,7 @@ void arrayline(char *line)
 	mis.super_minor = UnSet;
 	mis.level = UnSet;
 	mis.raid_disks = UnSet;
-	mis.spare_disks = UnSet;
+	mis.spare_disks = 0;
 	mis.devices = NULL;
 	mis.devname = NULL;
 	mis.spare_group = NULL;
@@ -364,32 +495,7 @@ void arrayline(char *line)
 				fprintf(stderr, Name ": metadata format %s unknown, ignored.\n", w+9);
 		} else if (strncasecmp(w, "auto=", 5) == 0 ) {
 			/* whether to create device special files as needed */
-			if (strcasecmp(w+5, "no")==0)
-				mis.autof = 0;
-			else if (strcasecmp(w+5,"yes")==0 || strcasecmp(w+5,"md")==0)
-				mis.autof = -1;
-			else {
-				/* There might be digits, and maybe a hyphen, at the end */
-				char *e = w+5 + strlen(w+5);
-				int num = 4;
-				int len;
-				while (e > w+5 && isdigit(e[-1]))
-					e--;
-				if (*e) {
-					num = atoi(e);
-					if (num <= 0) num = 1;
-				}
-				if (e > w+5 && e[-1] == '-')
-					e--;
-				len = e - (w+5);
-				if ((len == 3 && strncasecmp(w+5,"mdp",3)==0) ||
-				    (len == 1 && strncasecmp(w+5,"p",1)==0) ||
-				    (len >= 4 && strncasecmp(w+5,"part",4)==0))
-					mis.autof = num;
-				else 
-					fprintf(stderr, Name ": auto type of \"%s\" ignored for %s\n",
-						w+5, mis.devname?mis.devname:"unlabeled-array");
-			}
+			mis.autof = parse_auto(w+5, "auto type", 0);
 		} else {
 			fprintf(stderr, Name ": unrecognised word on ARRAY line: %s\n",
 				w);
@@ -423,6 +529,23 @@ void mailline(char *line)
 	}
 }
 
+static char *alert_mail_from = NULL;
+void mailfromline(char *line)
+{
+	char *w;
+
+	for (w=dl_next(line); w != line ; w=dl_next(w)) {
+		if (alert_mail_from == NULL)
+			alert_mail_from = strdup(w);
+		else {
+			char *t= NULL;
+			asprintf(&t, "%s %s", alert_mail_from, w);
+			free(alert_mail_from);
+			alert_mail_from = t;
+		}
+	}
+}
+
 
 static char *alert_program = NULL;
 void programline(char *line)
@@ -438,10 +561,30 @@ void programline(char *line)
 	}
 }
 
+static char *home_host = NULL;
+void homehostline(char *line)
+{
+	char *w;
+
+	for (w=dl_next(line); w != line ; w=dl_next(w)) {
+		if (home_host == NULL)
+			home_host = strdup(w);
+		else
+			fprintf(stderr, Name ": excess host name on HOMEHOST line: %s - ignored\n",
+				w);
+	}
+}
+
 
 int loaded = 0;
 
-void load_conffile(char *conffile)
+static char *conffile = NULL;
+void set_conffile(char *file)
+{
+	conffile = file;
+}
+
+void load_conffile(void)
 {
 	FILE *f;
 	char *line;
@@ -481,18 +624,27 @@ void load_conffile(char *conffile)
 	loaded = 1;
 	while ((line=conf_line(f))) {
 		switch(match_keyword(line)) {
-		case 0: /* DEVICE */
+		case Devices:
 			devline(line);
 			break;
-		case 1: /* ARRAY */
+		case Array:
 			arrayline(line);
 			break;
-		case 2: /* MAIL */
+		case Mailaddr:
 			mailline(line);
 			break;
-		case 3: /* PROGRAM */
+		case Mailfrom:
+			mailfromline(line);
+			break;
+		case Program:
 			programline(line);
 			break;
+		case CreateDev:
+			createline(line);
+			break;
+		case Homehost:
+			homehostline(line);
+			break;
 		default:
 			fprintf(stderr, Name ": Unknown keyword %s\n", line);
 		}
@@ -504,30 +656,47 @@ void load_conffile(char *conffile)
 /*    printf("got file\n"); */
 }
 
-char *conf_get_mailaddr(char *conffile)
+char *conf_get_mailaddr(void)
 {
-	load_conffile(conffile);
+	load_conffile();
 	return alert_email;
 }
 
-char *conf_get_program(char *conffile)
+char *conf_get_mailfrom(void)
 {
-	load_conffile(conffile);
+	load_conffile();
+	return alert_mail_from;
+}
+
+char *conf_get_program(void)
+{
+	load_conffile();
 	return alert_program;
 }
 
+char *conf_get_homehost(void)
+{
+	load_conffile();
+	return home_host;
+}
 
-mddev_ident_t conf_get_ident(char *conffile, char *dev)
+struct createinfo *conf_get_create_info(void)
+{
+	load_conffile();
+	return &createinfo;
+}
+
+mddev_ident_t conf_get_ident(char *dev)
 {
 	mddev_ident_t rv;
-	load_conffile(conffile);
+	load_conffile();
 	rv = mddevlist;
 	while (dev && rv && strcmp(dev, rv->devname)!=0)
 		rv = rv->next;
 	return rv;
 }
 
-mddev_dev_t conf_get_devs(char *conffile)
+mddev_dev_t conf_get_devs()
 {
 	glob_t globbuf;
 	struct conf_dev *cd;
@@ -542,8 +711,12 @@ mddev_dev_t conf_get_devs(char *conffile)
 		free(t);
 	}
     
-	load_conffile(conffile);
-    
+	load_conffile();
+
+	if (cdevlist == NULL)
+		/* default to 'partitions */
+		dlist = load_partitions();
+
 	for (cd=cdevlist; cd; cd=cd->next) {
 		if (strcasecmp(cd->name, "partitions")==0 && dlist == NULL)
 			dlist = load_partitions();
@@ -557,6 +730,7 @@ mddev_dev_t conf_get_devs(char *conffile)
 			mddev_dev_t t = malloc(sizeof(*t));
 			t->devname = strdup(globbuf.gl_pathv[i]);
 			t->next = dlist;
+			t->used = 0;
 			dlist = t;
 /*	printf("one dev is %s\n", t->devname);*/
 		}
